-- name: [SMASH CO-OP] Testma

-- char-template

SMASH_VERSION = 0.2

-- Hitbox Attachments 
local ATTACH_NONE       = 0
local ATTACH_HAND       = 1
local ATTACH_HEAD       = 2
local ATTACH_RELATIVE   = 3
local ATTACH_PROJECTILE = 4

-- Character Information
local charName          = "Custom Fighter"
local charWeight        = 1.1
local charHurtBoxSize   = 0.9
local charTopSpeed      = 40
local charAcceleration  = 4
local charDeceleration  = 2
local charFriction      = 1.5

enabled = false

if not _G.smash then
    djui_popup_create("Super Smash Bros. Co-op was not detected, please enable the mod to proceed", 2)
elseif _G.smash.SMASH_VERSION < SMASH_VERSION then
    djui_popup_create(string.format("%s Fighter Mod uses a newer version of Super Smash Bros. Co-op! Please update! (Fighter Mod version: %.2f)", stageName, SMASH_VERSION), 2)
else
    enabled = true
    gMarioStateExtras = _G.smash.gMarioStateExtras
    create_hitbox = _G.smash.create_hitbox
    stock_icon_render = _G.smash.stock_icon_render
    ACT_GROUND_POUND_A = _G.smash.ACT_GROUND_POUND_A
    ACT_GROUND_POUND_B = _G.smash.ACT_GROUND_POUND_B
end

texStockTemplate = get_texture_info("hud-stock-unknown")
texMenuCharTemplate = get_texture_info("menu-icon-unknown")

function convert_s16(num)
    local min = -32768
    local max = 32767
    while (num < min) do
        num = max + (num - min)
    end
    while (num > max) do
        num = min + (num - max)
    end
    return num
end

function template_mario_update(m)

end

function template_before_mario_update(m)

end

function template_stock_render(x, y, i, palTbl)
    stock_icon_render({{part = CAP, tex = texStockTemplate}}, i, x, y)
end

function template_on_hud_render()

end

function act_template_jab(m)
    local step = perform_ground_step(m)
    local e = gMarioStateExtras[m.playerIndex]

    m.marioObj.header.gfx.animInfo.animFrame = e.animFrame
    
    if (step == GROUND_STEP_NONE) then

        m.vel.x = m.vel.x * 0.8
        m.vel.z = m.vel.z * 0.8

        if (m.actionState == 0) then
            e.animFrame = 0
            m.actionState = 1
        elseif (m.actionState == 1) then -- Punch 1 Start -- Hitbox
            set_mario_animation(m, MARIO_ANIM_FIRST_PUNCH)
            e.animFrame = e.animFrame + 1

            -- Create Hitbox
            if e.animFrame == 2 then
                create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 0.6, 1.2, -6, -20, sins(m.faceAngle.y)*100, 75, coss(m.faceAngle.y)*100, 0, 0, 0, ATTACH_NONE, 5, 1)
            end

            if e.animFrame >= m.marioObj.header.gfx.animInfo.curAnim.loopEnd then
                m.actionState = 2
                e.animFrame = 0
            end
        elseif (m.actionState == 2) then -- Punch 1 End
            set_mario_animation(m, MARIO_ANIM_FIRST_PUNCH_FAST)
            e.animFrame = e.animFrame + 1

            -- Continue Attack
            if e.animFrame > 0 then
                if (m.controller.buttonPressed & B_BUTTON) ~= 0 then
                    m.actionState = 3
                    e.animFrame = 0
                end
            end
        elseif (m.actionState == 3) then -- Punch 2 Start -- Hitbox --
            set_mario_animation(m, MARIO_ANIM_SECOND_PUNCH)
            e.animFrame = e.animFrame + 1

            -- Create Hitbox
            if e.animFrame == 2 then
                create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 0.6, 1.2, 6, -20, sins(m.faceAngle.y)*100, 75, coss(m.faceAngle.y)*100, 0, 0, 0, ATTACH_NONE, 5, 1)
            end

            if e.animFrame >= m.marioObj.header.gfx.animInfo.curAnim.loopEnd then
                m.actionState = 4
                e.animFrame = 0
            end
            
        elseif (m.actionState == 4) then -- Punch 2 End
            set_mario_animation(m, MARIO_ANIM_SECOND_PUNCH_FAST)
            e.animFrame = e.animFrame + 1
            -- Continue Attack
            if e.animFrame > 0 then
                if (m.controller.buttonPressed & B_BUTTON) ~= 0 then
                    m.actionState = 5
                    e.animFrame = 0
                end
            end
        elseif (m.actionState == 5) then -- Kick End      -- Hitbox --
            set_mario_animation(m, MARIO_ANIM_GROUND_KICK)
            e.animFrame = e.animFrame + 1

            -- Create Hitbox
            if e.animFrame == 2 then
                create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 0.9, 3, 24, 40, sins(m.faceAngle.y)*100, 85, coss(m.faceAngle.y)*100, 0, 0, 0, ATTACH_NONE, 5, 1.2)
            end
        end

        if e.animFrame >= m.marioObj.header.gfx.animInfo.curAnim.loopEnd then
            set_mario_action(m, ACT_IDLE, 0)
        end

    elseif (step == GROUND_STEP_LEFT_GROUND) then
        set_mario_action(m, ACT_DOUBLE_JUMP, 2)
    end
end

function act_template_aerial(m)
    local step = perform_air_step(m, AIR_STEP_CHECK_LEDGE_GRAB)
    local e = gMarioStateExtras[m.playerIndex]

    m.marioObj.header.gfx.animInfo.animFrame = e.animFrame

    if m.actionTimer == 0 then
        m.vel.y = 16
    end
    m.actionTimer = m.actionTimer + 1
    
    if (step == AIR_STEP_NONE) then
        set_mario_animation(m, MARIO_ANIM_TWIRL)
        if m.actionTimer % 3 == 0 then
            create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 1, 1.2, m.forwardVel, 20, 0, 50, 0, 0, 0, 0, ATTACH_RELATIVE, 2, 1.1)
        end
        m.vel.y = m.vel.y + 1
        m.marioObj.header.gfx.angle.y = m.faceAngle.y+m.actionTimer*(13000-(m.actionTimer*420))+0x8000
        if m.actionTimer == 20 then
            set_mario_action(m, ACT_FREEFALL, 0)
        end
    elseif (step == AIR_STEP_LANDED) then
        set_mario_action(m, ACT_WALKING, 0)
    end
end

function act_template_ground_pound(m)
    local step = perform_air_step(m, AIR_STEP_CHECK_LEDGE_GRAB)
    local e = gMarioStateExtras[m.playerIndex]

    m.marioObj.header.gfx.animInfo.animFrame = e.animFrame

    m.actionTimer = m.actionTimer + 1
    
    if (step == AIR_STEP_NONE) then
        if (m.actionState == 0) then
            e.animFrame = 0
            m.actionState = 1
        elseif (m.actionState == 1) then
            set_mario_animation(m, MARIO_ANIM_START_GROUND_POUND)
            m.vel.x = m.vel.x * 0.7
            m.vel.y = 2
            m.vel.z = m.vel.z * 0.7
            e.animFrame = e.animFrame + (e.animFrame*0.2) + 0.5
            if e.animFrame >= m.marioObj.header.gfx.animInfo.curAnim.loopEnd then
                m.actionState = 2
                m.actionTimer = 0
            end
        elseif (m.actionState == 2) then
            -- Create Hitbox
            create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 1.1, 8.6, 32, 280, 0, -50, 0, 0, 0, 0, ATTACH_RELATIVE, 6, 1.5)
            m.actionState = 3
        elseif (m.actionState == 3) then
            m.vel.x = 0
            m.vel.y = -72
            m.vel.z = 0
            -- Create Hitbox
            if m.actionTimer == 2 then
                --create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 0.8, 2.4, 32, -80, 0, -70, 0, 0, 0, 0, ATTACH_RELATIVE, 12, 1.2)
            end
            m.actionState = 3
        end

        if (m.controller.buttonPressed & A_BUTTON) ~= 0 and e.groundPoundA then
            e.groundPoundA = false
            set_mario_action(m, ACT_GROUND_POUND_A, 0)--gCharData[m.character.type].groundPoundA(m)
        end

        if (m.controller.buttonPressed & B_BUTTON) ~= 0 then
            set_mario_action(m, ACT_GROUND_POUND_B, 0)--gCharData[m.character.type].groundPoundB(m)
        end

    elseif (step == AIR_STEP_LANDED) then
        set_mario_action(m, ACT_GROUND_POUND_LAND, 0)
        m.particleFlags = m.particleFlags | PARTICLE_HORIZONTAL_STAR
    end
end

function act_template_ground_pound_a(m)
    local e = gMarioStateExtras[m.playerIndex]
    set_mario_action(m, ACT_DOUBLE_JUMP, 0)
    m.faceAngle.y = m.intendedYaw
end

function act_template_ground_pound_b(m)
    local step = perform_air_step(m, AIR_STEP_CHECK_LEDGE_GRAB)

    local e = gMarioStateExtras[m.playerIndex]

    --m.marioObj.header.gfx.animInfo.animFrame = e.animFrame
    if m.actionTimer == 0 then
        m.vel.y = 8
    end
    m.actionTimer = m.actionTimer + 1
    if (step == AIR_STEP_NONE or step == AIR_STEP_HIT_WALL) then
        if m.actionState == 0 then
            if (m.controller.buttonDown & B_BUTTON) == 0 or m.actionTimer > 60 then
                set_mario_animation(m, MARIO_ANIM_CROUCHING)
                m.marioObj.header.gfx.animInfo.animFrame = 0
                mario_set_forward_vel(m, 48+(m.actionTimer))
                m.vel.x = m.vel.x * 0.9
                m.vel.y = 40
                m.vel.z = m.vel.z * 0.9

                create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 1.1, m.forwardVel*0.3, m.forwardVel, 15, sins(m.faceAngle.y)*20, 50, coss(m.faceAngle.y)*20, 0, 0, 0, ATTACH_RELATIVE, m.actionTimer*0.5, 0)

                m.actionTimer = 0
                m.actionState = 1
            else
                m.faceAngle.y = m.intendedYaw
                m.vel.x = m.vel.x * 0.9
                if m.vel.y < -32 then
                    m.vel.y = m.vel.y + 4
                else 
                    m.vel.y = m.vel.y + 3
                end
                m.vel.z = m.vel.z * 0.9
            end
        elseif m.actionState == 1 then
            set_mario_anim_with_accel(m, MARIO_ANIM_FLY_FROM_CANNON, 0x800 * m.forwardVel)
            
            mario_set_forward_vel(m, m.forwardVel*0.95)
            m.vel.y = m.vel.y + 3

            if m.forwardVel < 20 then
                set_mario_action(m, ACT_FREEFALL, 0)
            end
        end
    elseif (step == AIR_STEP_LANDED) then
        if m.actionState == 0 then
            if (m.controller.buttonDown & B_BUTTON) == 0 or m.actionTimer > 60 then
                set_mario_animation(m, MARIO_ANIM_CROUCHING)
                m.marioObj.header.gfx.animInfo.animFrame = 0
                mario_set_forward_vel(m, 48+(m.actionTimer))
                m.vel.y = 40

                create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 1.1, m.forwardVel*0.3, m.forwardVel, 15, sins(m.faceAngle.y)*20, 50, coss(m.faceAngle.y)*20, 0, 0, 0, ATTACH_RELATIVE, m.actionTimer*0.5, 0)

                m.actionTimer = 0
                m.actionState = 1
            else
                m.faceAngle.y = m.intendedYaw
                m.vel.x = m.vel.x * 0.9
                m.vel.z = m.vel.z * 0.9
            end
        else
            set_mario_action(m, ACT_FORWARD_ROLLOUT, 0)
        end
    end
end

function act_template_slide_kick(m)
    local step = perform_air_step(m, AIR_STEP_CHECK_LEDGE_GRAB)
    local e = gMarioStateExtras[m.playerIndex]

    if (m.actionTimer == 0) then
        
        mario_set_forward_vel(m, 40)
    end
    m.marioObj.header.gfx.animInfo.animFrame = e.animFrame

    m.actionTimer = m.actionTimer + 1
    
    if (step == AIR_STEP_NONE) then
        set_mario_animation(m, MARIO_ANIM_SLIDE_KICK)
        e.animFrame = e.animFrame + 1
        if (m.actionTimer % 3 == 0) then
            create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 0.6, 2.4, 46, 10, sins(m.faceAngle.y)*140, 50, coss(m.faceAngle.y)*140, 0, 0, 0, ATTACH_RELATIVE, 2, 0)
        end
    elseif (step == AIR_STEP_LANDED) then
        set_mario_action(m, ACT_FORWARD_ROLLOUT, 0)
    end
end

function act_template_dash(m)
    local step = perform_ground_step(m)
    local e = gMarioStateExtras[m.playerIndex]

    m.marioObj.header.gfx.animInfo.animFrame = e.animFrame
    m.actionTimer = m.actionTimer + 1
    if (step == GROUND_STEP_NONE or step == GROUND_STEP_HIT_WALL) then
        if m.actionState == 0 then
            mario_set_forward_vel(m, 60)
            set_mario_animation(m, MARIO_ANIM_WALK_WITH_LIGHT_OBJ)
            m.actionState = 1
            
        elseif m.actionState == 1 then
            m.marioObj.header.gfx.angle.y = m.faceAngle.y+sins(m.actionTimer*0x4000)*0x1500---0x2000*m.actionTimer
            m.marioObj.header.gfx.angle.x = 0x1000
            mario_set_forward_vel(m, m.forwardVel * 0.95)
            e.animFrame = e.animFrame + 16
            if m.marioObj.header.gfx.animInfo.animFrame >= m.marioObj.header.gfx.animInfo.curAnim.loopEnd then
                e.animFrame = e.animFrame - m.marioObj.header.gfx.animInfo.curAnim.loopEnd
            end
            if m.actionTimer % 4 == 0 then
                create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 1.1, 0.6, m.forwardVel, 30, sins(m.faceAngle.y)*120, 50, coss(m.faceAngle.y)*120, 0, 0, 0, ATTACH_RELATIVE, 2, 0)
            end
            if m.actionTimer > 25 then
                create_hitbox(m, E_MODEL_NONE, SOUND_ACTION_BONK, 1.2, 8.2, 32, 55, sins(m.faceAngle.y)*150, 50, coss(m.faceAngle.y)*150, 0, 0, 0, ATTACH_RELATIVE, 4, 1.1)
                m.actionState = 2
                e.animFrame = 0
            end
            m.faceAngle.y = m.intendedYaw - approach_s32(convert_s16(m.intendedYaw - m.faceAngle.y), 0, 0x200, 0x200)
        elseif m.actionState == 2 then
            set_mario_animation(m, MARIO_ANIM_WALK_PANTING)
            mario_set_forward_vel(m, m.forwardVel * 0.8)
            local animSpeed = 1
            
            e.animFrame = e.animFrame + animSpeed
            if e.animFrame >= m.marioObj.header.gfx.animInfo.curAnim.loopEnd then
                set_mario_action(m, ACT_IDLE, 0)
            end
        end
    elseif (step == GROUND_STEP_LEFT_GROUND) then
        m.vel.y = 0
        set_mario_action(m, ACT_FREEFALL, 0)
    end
end

palTemplate = {
    [1] = {
        name = "Base",
        [PANTS] = 0xF5F0FF,     -- Head
        [SHIRT] = 0xF5F0FF,     -- Body/Arms
        [GLOVES] = 0xF5F0FF,    -- ????
        [SHOES] = 0xF5F0FF,     -- ????
        [HAIR] = 0xF5F0FF,      -- Hair
        [SKIN] = 0xF5F0FF,      -- Skin
        [CAP] = 0xF5F0FF,       -- Spikes???
        [EMBLEM] = 0x212121,
    },
}

if enabled then
    _G.smash.smash_add_character(
        charName,
        charWeight,                 -- Weight (How much a part of knockback affects the character)
        charHurtBoxSize,            -- Size of hurtbox
        charTopSpeed,               -- Top Speed
        charAcceleration,           -- Acceleration
        charDeceleration,           -- Deceleration
        charFriction,               -- Friction
        texMenuCharTemplate,
        E_MODEL_MARIO,              -- Character Model
        CT_MARIO,                   -- Character Override
        template_mario_update,         -- Character Mario Update
        template_before_mario_update,  -- Character Before Mario Update
        template_stock_render,         -- Character Stock Icon Render
        template_on_hud_render,
        -- Actions
        act_template_jab,
        act_template_aerial,
        act_template_ground_pound,
        act_template_ground_pound_a,
        act_template_ground_pound_b,
        act_template_slide_kick,
        act_template_dash,
        palTemplate
    )
end